/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.user;

import java.util.Arrays;
import java.util.Optional;

import org.keycloak.admin.client.CreatedResponseUtil;
import org.keycloak.admin.client.Keycloak;
import org.keycloak.admin.client.resource.RealmResource;
import org.keycloak.representations.idm.ClientRepresentation;
import org.keycloak.representations.idm.RoleRepresentation;
import org.keycloak.representations.idm.UserRepresentation;
import org.springframework.stereotype.Component;

import de.ozgcloud.operator.keycloak.KeycloakException;
import de.ozgcloud.operator.keycloak.KeycloakGenericRemoteService;
import de.ozgcloud.operator.keycloak.KeycloakResultParser;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Component
class KeycloakUserRemoteService {

	private final Keycloak keycloak;

	private final KeycloakGenericRemoteService keycloakGenericRemoteService;

	public void createUser(UserRepresentation user, String namespace) {
		var realmResource = getRealm(namespace);

		var response = realmResource.users().create(user);
		KeycloakResultParser.parseCreatedResponse(response);

		var userId = CreatedResponseUtil.getCreatedId(response);

		addClientRoles(userId, realmResource, namespace, user);
	}

	public Optional<UserRepresentation> getUserByName(String userName, String namespace) {
		return getRealm(namespace).users().search(userName, true).stream().findFirst();
	}

	public void deleteUser(String userId, String namespace) {
		getRealm(namespace).users().delete(userId);
	}

	public void updateUser(UserRepresentation user, String namespace) {
		var realmResource = getRealm(namespace);

		realmResource.users().get(user.getId()).update(user);

		addClientRoles(user.getId(), realmResource, namespace, user);
	}

	private RealmResource getRealm(String namespace) {
		return keycloak.realm(namespace);
	}

	void addClientRoles(String userId, RealmResource realmResource, String namespace, UserRepresentation user) {
		user.getClientRoles().keySet().forEach(clientId -> {
			var realmClient = getRealmClient(realmResource, clientId);
			user.getClientRoles().get(clientId).stream()
					.map(clientRoleName -> keycloakGenericRemoteService.getClientRole(clientRoleName, realmClient.getId(), namespace)
							.orElseThrow(() -> new KeycloakException(
									"Role " + clientRoleName + " not found for client with clientId " + clientId + " in realm " + namespace)))
					.forEach(clientRole -> addClientRoleToUser(clientRole, realmResource, userId, realmClient));
		});
	}

	ClientRepresentation getRealmClient(RealmResource realmResource, String clientId) {
		return realmResource.clients()
				.findByClientId(clientId).stream().findFirst()
				.orElseThrow(() -> new KeycloakException("Could not find Client with cliendId " + clientId));
	}

	void addClientRoleToUser(RoleRepresentation clientRole, RealmResource realmResource, String userId, ClientRepresentation appClient) {
		realmResource.users().get(userId).roles().clientLevel(appClient.getId()).add(Arrays.asList(clientRole));
	}
}